from os.path import dirname

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = (
    'DEBUG',
    'NAMES_LOCAL_PATH_PARENT',
)

# Absolute base path that is prepended to NAMES_LOCAL_PATH
NAMES_LOCAL_PATH_PARENT = dirname(__file__)

DEBUG = False



# -*- coding: utf-8 -*-

import logging
import unittest

import subprocess

from .base import log_info, internet_available_only

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('TestCommands',)

LOGGER = logging.getLogger(__name__)


class TestCommands(unittest.TestCase):
    """Tld commands tests."""

    def setUp(self):
        """Set up."""

    @internet_available_only
    @log_info
    def test_1_update_tld_names_command(self):
        """Test updating the tld names (re-fetch mozilla source)."""
        res = subprocess.check_output(['update-tld-names']).strip()
        self.assertEqual(res, b'')
        return res

    @internet_available_only
    @log_info
    def test_1_update_tld_names_mozilla_command(self):
        """Test updating the tld names (re-fetch mozilla source)."""
        res = subprocess.check_output(['update-tld-names', 'mozilla']).strip()
        self.assertEqual(res, b'')
        return res


if __name__ == '__main__':
    unittest.main()



from typing import Dict
from urllib.parse import SplitResult

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = (
    'Result',
)


class Result(object):
    """Container."""

    __slots__ = ('subdomain', 'domain', 'tld', '__fld', 'parsed_url')

    def __init__(self,
                 tld: str,
                 domain: str,
                 subdomain: str,
                 parsed_url: SplitResult):
        self.tld = tld
        self.domain = domain if domain != '' else tld
        self.subdomain = subdomain
        self.parsed_url = parsed_url

        if domain:
            self.__fld = f"{self.domain}.{self.tld}"
        else:
            self.__fld = self.tld

    @property
    def extension(self) -> str:
        """Alias of ``tld``.

        :return str:
        """
        return self.tld
    suffix = extension

    @property
    def fld(self) -> str:
        """First level domain.

        :return:
        :rtype: str
        """
        return self.__fld

    def __str__(self) -> str:
        return self.tld
    __repr__ = __str__

    @property
    def __dict__(self) -> Dict[str, str]:
        """Mimic __dict__ functionality.

        :return:
        :rtype: dict
        """
        return {
            'tld': self.tld,
            'domain': self.domain,
            'subdomain': self.subdomain,
            'fld': self.fld,
            'parsed_url': self.parsed_url,
        }



from os.path import abspath, join

from .conf import get_setting

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = (
    'project_dir',
    'PROJECT_DIR',
)


def project_dir(base: str) -> str:
    """Project dir."""
    tld_names_local_path_parent = get_setting('NAMES_LOCAL_PATH_PARENT')
    return abspath(
        join(tld_names_local_path_parent, base).replace('\\', '/')
    )


PROJECT_DIR = project_dir



from typing import Type, Dict

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = (
    'Registry',
)


class Registry(type):

    REGISTRY = {}

    def __new__(cls, name, bases, attrs):
        new_cls = type.__new__(cls, name, bases, attrs)
        # Here the name of the class is used as key but it could be any class
        # parameter.
        if getattr(new_cls, '_uid', None):
            cls.REGISTRY[new_cls._uid] = new_cls
        return new_cls

    @property
    def _uid(cls) -> str:
        return getattr(cls, 'uid', cls.__name__)

    @classmethod
    def reset(cls) -> None:
        cls.REGISTRY = {}

    @classmethod
    def get(cls, key, default=None):
        return cls.REGISTRY.get(key, default)

    @classmethod
    def items(cls):
        return cls.REGISTRY.items()

    # @classmethod
    # def get_registry(cls) -> Dict[str, Type]:
    #     return dict(cls.REGISTRY)
    #
    # @classmethod
    # def pop(cls, uid) -> None:
    #     cls.REGISTRY.pop(uid)



# -*- coding: utf-8 -*-

from functools import lru_cache
import logging
import socket

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = (
    'internet_available_only',
    'log_info',
)

LOG_INFO = True
LOGGER = logging.getLogger(__name__)


def log_info(func):
    """Log some useful info."""
    if not LOG_INFO:
        return func

    def inner(self, *args, **kwargs):
        """Inner."""
        result = func(self, *args, **kwargs)

        LOGGER.debug('\n\n%s', func.__name__)
        LOGGER.debug('============================')
        if func.__doc__:
            LOGGER.debug('""" %s """', func.__doc__.strip())
        LOGGER.debug('----------------------------')
        if result is not None:
            LOGGER.debug(result)
        LOGGER.debug('\n++++++++++++++++++++++++++++')

        return result
    return inner


@lru_cache(maxsize=32)
def is_internet_available(host="8.8.8.8", port=53, timeout=3):
    """Check if internet is available.

    Host: 8.8.8.8 (google-public-dns-a.google.com)
    OpenPort: 53/tcp
    Service: domain (DNS/TCP)
    """
    try:
        socket.setdefaulttimeout(timeout)
        socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect((host, port))
        return True
    except socket.error as ex:
        print(ex)
        return False


def internet_available_only(func):

    def inner(self, *args, **kwargs):
        """Inner."""
        if not is_internet_available():
            LOGGER.debug('\n\n%s', func.__name__)
            LOGGER.debug('============================')
            if func.__doc__:
                LOGGER.debug('""" %s """', func.__doc__.strip())
            LOGGER.debug('----------------------------')
            LOGGER.debug("Skipping because no Internet connection available.")
            LOGGER.debug('\n++++++++++++++++++++++++++++')
            return None

        result = func(self, *args, **kwargs)
        return result

    return inner



from .conf import get_setting

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = (
    'TldBadUrl',
    'TldDomainNotFound',
    'TldImproperlyConfigured',
    'TldIOError',
)


class TldIOError(IOError):
    """TldIOError.

    Supposed to be thrown when problems with reading/writing occur.
    """

    def __init__(self, msg=None):
        tld_names_local_path = get_setting('NAMES_LOCAL_PATH')
        if msg is None:
            msg = "Can't read from or write to the %s " \
                  "file!" % tld_names_local_path
        super(TldIOError, self).__init__(msg)


class TldDomainNotFound(ValueError):
    """TldDomainNotFound.

    Supposed to be thrown when domain name is not found (didn't match) the
    local TLD policy.
    """

    def __init__(self, domain_name):
        super(TldDomainNotFound, self).__init__(
            "Domain %s didn't match any existing TLD name!" % domain_name
        )


class TldBadUrl(ValueError):
    """TldBadUrl.

    Supposed to be thrown when bad URL is given.
    """

    def __init__(self, url):
        super(TldBadUrl, self).__init__("Is not a valid URL %s!" % url)


class TldImproperlyConfigured(Exception):
    """TldImproperlyConfigured.

    Supposed to be thrown when code is improperly configured. Typical use-case
    is when user tries to use `get_tld` function with both `search_public` and
    `search_private` set to False.
    """

    def __init__(self, msg=None):
        if msg is None:
            msg = "Improperly configured."
        else:
            msg = "Improperly configured. %s" % msg

        super(TldImproperlyConfigured, self).__init__(msg)



from codecs import open as codecs_open
from urllib.request import urlopen

from .exceptions import (
    TldIOError,
    TldImproperlyConfigured,
)
from .helpers import project_dir
from .registry import Registry

__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = ('BaseTLDSourceParser',)


class BaseTLDSourceParser(metaclass=Registry):
    """Base TLD source parser."""

    uid: str = None
    source_url: str = None
    local_path: str = None

    @classmethod
    def validate(cls):
        """Constructor."""
        if not cls.uid:
            raise TldImproperlyConfigured(
                "The `uid` property of the TLD source parser shall be defined."
            )

    @classmethod
    def get_tld_names(cls, fail_silently: bool = False, retry_count: int = 0):
        """Get tld names.

        :param fail_silently:
        :param retry_count:
        :return:
        """
        cls.validate()
        raise NotImplementedError(
            "Your TLD source parser shall implement `get_tld_names` method."
        )

    @classmethod
    def update_tld_names(cls, fail_silently: bool = False) -> bool:
        """Update the local copy of the TLD file.

        :param fail_silently:
        :return:
        """
        try:
            remote_file = urlopen(cls.source_url)
            local_file = codecs_open(
                project_dir(cls.local_path),
                'wb',
                encoding='utf8'
            )
            local_file.write(remote_file.read().decode('utf8'))
            local_file.close()
            remote_file.close()
        except Exception as err:
            if fail_silently:
                return False
            raise TldIOError(err)

        return True



from .utils import (
    get_fld,
    get_tld,
    get_tld_names,
    is_tld,
    parse_tld,
    Result,
    update_tld_names,
)

__title__ = 'tld'
__version__ = '0.11.10'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later'
__all__ = (
    'get_fld',
    'get_tld',
    'get_tld_names',
    'is_tld',
    'parse_tld',
    'Result',
    'update_tld_names',
)



import os
import sys
from setuptools import setup, find_packages

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
    readme = ''

version = '0.11.11'

py_where = './src'
py_package_dir = 'src'
try:
    if sys.argv[-2] == '--python-tag':
        # For Python 3.5
        if sys.argv[-1] == 'py35':
            py_where = './src_py35'
            py_package_dir = 'src_py35'

        # For Python 2.7
        elif sys.argv[-1] == 'py27':
            py_where = './src_py27'
            py_package_dir = 'src_py27'

        # For development mode
        if sys.argv[1] in ('develop', 'install',):
            sys.argv.pop(-1)
            sys.argv.pop(-1)
except Exception as err:
    pass

setup(
    name='tld',
    version=version,
    description="Extract the top-level domain (TLD) from the URL given.",
    long_description=readme,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Internet",
        "License :: OSI Approved :: Mozilla Public License 1.1 (MPL 1.1)",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or "
        "later (LGPLv2+)",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/barseghyanartur/tld/issues",
        "Documentation": "https://tld.readthedocs.io/",
        "Source Code": "https://github.com/barseghyanartur/tld/",
        "Changelog": "https://tld.readthedocs.io/en/latest/changelog.html",
    },
    python_requires=">=2.7, <4",
    keywords='tld, top-level domain names, python',
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    url='https://github.com/barseghyanartur/tld',
    package_dir={'': py_package_dir},
    packages=find_packages(where=py_where),
    entry_points={
        'console_scripts': [
            'update-tld-names = tld.utils:update_tld_names_cli'
        ]
    },
    include_package_data=True,
    license='MPL-1.1 OR GPL-2.0-only OR LGPL-2.1-or-later',
    install_requires=[
        'six;python_version<="3.5"',  # Used in Python 2.7 and 3.5 dist
        'typing;python_version<"3.5"',  # Used in Python < 3.5 dist
        'backports.functools-lru-cache;python_version<"3.5"',  # For Python 2.7
    ],
    test_suite='tld.tests',
    tests_require=[
        'coverage',
        'factory_boy',
        'Faker',
        'pytest-cov',
        'pytest',
        'tox',
    ]
)
